#include <stdio.h>
#include <stdlib.h>

#include <glib.h>

#include <sys/resource.h>

#ifndef NUM_SWITCHES
#define NUM_SWITCHES 100
#endif

GMutex *mutex;
GCond *main_cond, *thread_cond;
gboolean in_thread;


void
get_times (float *user, float *system)
{
  struct rusage rusage;

  getrusage (RUSAGE_SELF, &rusage);
  *user = rusage.ru_utime.tv_sec + rusage.ru_utime.tv_usec / 1000000.0;
  *system = rusage.ru_stime.tv_sec + rusage.ru_stime.tv_usec / 1000000.0;
}

gpointer
thread_func (gpointer data)
{
  int i;

  for (i = 0; i < NUM_SWITCHES; i++) {
    //printf ("In thread\n");

    g_mutex_lock (mutex);

    while (!in_thread) {
      g_cond_wait (thread_cond, mutex);
    }

    in_thread = FALSE;
    g_cond_signal (main_cond);

    g_mutex_unlock (mutex);
  }

  return NULL;
}


int
main (void)
{
  GThread *thread;
  int i;
  float iu, is, fu, fs;

  g_thread_init (NULL);

  mutex = g_mutex_new ();

  main_cond = g_cond_new ();
  thread_cond = g_cond_new ();

  in_thread = FALSE;

  thread = g_thread_create (thread_func, NULL, FALSE, NULL);

  get_times (&iu, &is);

  for (i = 0; i < NUM_SWITCHES; i++) {
    //printf ("In main\n");

    g_mutex_lock (mutex);

    while (in_thread) {
      g_cond_wait (main_cond, mutex);
    }

    in_thread = TRUE;
    g_cond_signal (thread_cond);

    g_mutex_unlock (mutex);
  }

  get_times (&fu, &fs);

  printf ("%0.3f %0.3f\n", fu - iu, fs - is);

  return 0;
}
