#include <stdio.h>
#include <stdlib.h>

#include <pth.h>

#include <sys/resource.h>

#define PTH_CALL(func, args...) \
  if (!func (args)) { \
    fprintf (stderr, #func " failed\n"); \
    exit (1); \
  }

#ifndef NUM_SWITCHES
#define NUM_SWITCHES 100
#endif

pth_uctx_t uctx_main, uctx_cth;

void
get_times (float *user, float *system)
{
  struct rusage rusage;

  getrusage (RUSAGE_SELF, &rusage);
  *user = rusage.ru_utime.tv_sec + rusage.ru_utime.tv_usec / 1000000.0;
  *system = rusage.ru_stime.tv_sec + rusage.ru_stime.tv_usec / 1000000.0;
}

void 
cothread_func (void *arg)
{
  int i;

  for (i = 0; i < NUM_SWITCHES; i++) {
    //printf ("In cothread\n");
    PTH_CALL (pth_uctx_switch, uctx_cth, uctx_main);
  }
}

int
main (void)
{
  int i;
  float iu, is, fu, fs;

  PTH_CALL (pth_init);

  /* Create the main context. */
  PTH_CALL (pth_uctx_create, &uctx_main);

  /* Create the cothread context. */
  PTH_CALL (pth_uctx_create, &uctx_cth);
  PTH_CALL (pth_uctx_make, uctx_cth, NULL, 32 * 1024, NULL,
      cothread_func, NULL, uctx_main);

  get_times (&iu, &is);

  for (i = 0; i < NUM_SWITCHES; i++) {
    //printf ("In main\n");
    PTH_CALL (pth_uctx_switch, uctx_main, uctx_cth);
  }

  get_times (&fu, &fs);

  PTH_CALL (pth_uctx_destroy, uctx_main);
  PTH_CALL (pth_uctx_destroy, uctx_cth);

  PTH_CALL (pth_kill);

  printf ("%0.3f %0.3f\n", fu - iu, fs - is);

  return 0;
}
